///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 1995 - 2000 OSR Open Systems Resources, Inc.
//    All Rights Reserved
//
//    This sofware is supplied for instructional purposes only.
//
//    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
//    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
//    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
//    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
//    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
//    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
//    exceed the price paid for this material.  In no event shall OSR or its
//    suppliers be liable for any damages whatsoever (including, without
//    limitation, damages for loss of business profit, business interruption,
//    loss of business information, or any other pecuniary loss) arising out
//    of the use or inability to use this software, even if OSR has been
//    advised of the possibility of such damages.  Because some states/
//    jurisdictions do not allow the exclusion or limitation of liability for
//    consequential or incidental damages, the above limitation may not apply
//    to you.
//
//    OSR Open Systems Resources, Inc.
//    105 Route 101A Suite 19
//    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//    email bugs to: bugs@osr.com
//
//
//    MODULE:
//
//        NothingIO.C -- Nothing driver V2 - Read/Write/Ioctl functions
//
//    ABSTRACT:
//
//      This file contains the skeleton code for a WDM/Win2K device driver
//      that does absolutely nothing.
//
//    AUTHOR(S):
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
//        This is a WDM compatible version of the original NT V4
//        NOTHING driver.
//
///////////////////////////////////////////////////////////////////////////////


#include <ntddk.h>
#include "nothing.h"

///////////////////////////////////////////////////////////////////////////////
//
//  NothingCreateClose
//
//    This is the dispatch entry point for processing both IRP_MJ_CREATE
//    and IRP_MJ_CLOSE functions.  Since this is a simple device driver,
//    there's really nothing to do here by complete the requests with success.
//
//
//  INPUTS:
//
//    DeviceObject - Address of the DEVICE_OBJECT for our device.
//  
//    Irp - Address of the IRP representing the IRP_MJ_CREATE or IRP_MJ_CLOSE
//          call.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//    STATUS_SUCCESS.   We never fail this function.        
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingCreateClose(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{

#if DBG
    DbgPrint(DRIVER_NAME "CreateClose: Starting\n");
#endif

    //
    // Nothing much to do....
    //
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information = 0;

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#if DBG
    DbgPrint(DRIVER_NAME "CreateClose: Leaving\n");
#endif

    return(STATUS_SUCCESS);
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingWrite
//
//    This is the write dispatch entry point for the driver, called when the
//    I/O Manager has an IRP_MJ_WRITE request for the driver to process.
//
//  INPUTS:
//
//      DeviceObject - Address of the DEVICE_OBJECT for our device.
//  
//    Irp - Address of the IRP representing the IRP_MJ_WRITE call.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//    STATUS_PENDING, since we are putting the IRP on our internal queue.
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
//    Since we set the DO_BUFFERED_IO bit in the Device Object, all buffers
//    passed to us reside in the system's non-paged pool, a pointer to this
//    buffer is in Irp->AssociatedIrp.SystemBuffer.
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingWrite(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    PIO_STACK_LOCATION ioStack = IoGetCurrentIrpStackLocation(Irp);

#if DBG
    DbgPrint(DRIVER_NAME "Write: Starting\n");
#endif

    //
    // Complete the IRP with success
    //
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information =  ioStack->Parameters.Write.Length;

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#if DBG
    DbgPrint(DRIVER_NAME "Write: Leaving\n");
#endif

    return(STATUS_SUCCESS);
}

///////////////////////////////////////////////////////////////////////////////
//
//  NothingRead
//
//    This is the Read dispatch entry point for the driver, called when the
//    I/O Manager has an IRP_MJ_READ request for the driver to process.
//
//  INPUTS:
//
//    DeviceObject - Address of the DEVICE_OBJECT for our device.
//  
//    Irp - Address of the IRP representing the IRP_MJ_READ call.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//    STATUS_PENDING, since we are putting the IRP on our internal queue.
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//    Since we set the DO_BUFFERED_IO bit in the Device Object, all buffers
//    passed to us reside in the system's non-paged pool, a pointer to this
//    buffer is in Irp->AssociatedIrp.SystemBuffer.
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingRead(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    PIO_STACK_LOCATION ioStack = IoGetCurrentIrpStackLocation(Irp);

#if DBG
    DbgPrint(DRIVER_NAME "Read: Starting\n");
#endif

    //
    // Complete the IRP with success
    //
    Irp->IoStatus.Status = STATUS_SUCCESS;
    Irp->IoStatus.Information =  ioStack->Parameters.Read.Length;

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#if DBG
    DbgPrint(DRIVER_NAME "Read: Leaving\n");
#endif

    return(STATUS_SUCCESS);
}


///////////////////////////////////////////////////////////////////////////////
//
//  NothingDeviceControl
//
//    This is the dispatch entry point for processing IRP_MJ_DEVICE_CONTROL
//    Irps.  
//
//
//  INPUTS:
//
//    DeviceObject - Address of the DEVICE_OBJECT for our device.
//  
//    Irp - Address of the IRP representing the IRP_MJ_DEVICE_CONTROL call.
//
//  OUTPUTS:
//
//      None.
//
//  RETURNS:
//
//    STATUS_SUCCESS if success, otherwise an appropriate status is returned.
//
//  IRQL:
//
//    This routine is called at IRQL_PASSIVE_LEVEL.
//
//  NOTES:
//
//    We presently only support IOCTL_OSR_NOTHING.
//
///////////////////////////////////////////////////////////////////////////////
NTSTATUS NothingDeviceControl(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
    ULONG operation;
    PIO_STACK_LOCATION ioStack = IoGetCurrentIrpStackLocation(Irp);
    NTSTATUS code;

#if DBG
    DbgPrint(DRIVER_NAME "DeviceControl: Starting\n");
#endif

    operation = ioStack->Parameters.DeviceIoControl.IoControlCode;

    //
    // Set the status field to zero for completion
    //
    Irp->IoStatus.Information = 0;

    switch (operation) {

        case IOCTL_OSR_NOTHING:

            code = STATUS_SUCCESS;
#if DBG
            DbgPrint(DRIVER_NAME "DeviceControl: IOCTL_OSR_NOTHING processed\n");
#endif

            break;            


        default:

            //
            // This is the case for a bogus request code.
            //
            code = STATUS_INVALID_PARAMETER;

#if DBG
            DbgPrint(DRIVER_NAME "DeviceControl: Invalid IOCTL code 0x%0x\n", operation);
#endif

            break;
    }

    //
    // Complete the I/O Request
    //
    Irp->IoStatus.Status = code;

    IoCompleteRequest(Irp, IO_NO_INCREMENT);

#if DBG
    DbgPrint(DRIVER_NAME "DeviceControl: Leaving\n");
#endif

    return(code);
}
